/*	Sample program for Olimex AVR-MT-128 with ATMega-12 8 processor
 *	Hold Button 3 (Middle Button) to activate the Buzzer or start blink the led
 *	Button1/Button5 (Up/Down Button) to switch the action (blinking/buzzing)
 *	Button2/Button4 (Left/Right Button) to decrease/increase the frequency of the buzzer or speed of the relay
 *	Compile with AVRStudio+WinAVR (gcc version 3.4.6)
 */

#include "avr/io.h"
#include "lcd.h"

#define	__AVR_ATMEGA128__	1

unsigned char data;
unsigned int Bl = 1, Line = 2, Freq[2];

void Delay(unsigned int a)
{
 	while (a) 
	{
		a--;
	}
}

/**********************************************************L C D*******************************************************/

void E_Pulse()
{
	PORTC = PORTC | 0b00000100;	//set E to high
	Delay(1400);
	PORTC = PORTC & 0b11111011;	//set E to low
}

void LCD_Init()
{
	//LCD initialization
	//step by step (from Gosho) - from DATASHEET

	PORTC = PORTC & 0b11111110;				//set RS port to 0
	
	Delay(10000);


	PORTC = 0b00110000;						//set D4, D5 port to 1
	E_Pulse();								//high->low to E port (pulse)
	Delay(1000);

	PORTC = 0b00110000;						//set D4, D5 port to 1
	E_Pulse();								//high->low to E port (pulse)
	Delay(1000);

	PORTC = 0b00110000;						//set D4, D5 port to 1
	E_Pulse();								//high->low to E port (pulse)
	Delay(1000);

	PORTC = 0b00100000;						//set D4 to 0, D5 port to 1
	E_Pulse();								//high->low to E port (pulse)
}

void LCDSendCommand(unsigned char a)
{
	data = 0b00001111 | a;					//get high 4 bits
	PORTC = (PORTC | 0b11110000) & data;	//set D4-D7
	PORTC = PORTC & 0b11111110;				//set RS port to 0
	E_Pulse();                              //pulse to set D4-D7 bits

	data = a<<4;							//get low 4 bits
	PORTC = (PORTC & 0b00001111) | data;	//set D4-D7
	PORTC = PORTC & 0b11111110;				//set RS port to 0 -> display set to command mode
	E_Pulse();                              //pulse to set d4-d7 bits

}

void LCDSendChar(unsigned char a)
{
	data = 0b00001111 | a;					//get high 4 bits
	PORTC = (PORTC | 0b11110000) & data;	//set D4-D7
	PORTC = PORTC | 0b00000001;				//set RS port to 1
	E_Pulse();                              //pulse to set D4-D7 bits

	data = a<<4;							//get low 4 bits
	PORTC = (PORTC & 0b00001111) | data;	//clear D4-D7
	PORTC = PORTC | 0b00000001;				//set RS port to 0 -> display set to command mode
	E_Pulse();                              //pulse to set d4-d7 bits
}

void LCDSendTxt(char* a)
{
	int Temp;
	for(Temp=0; Temp<strlen(a); Temp++) 
		LCDSendChar(a[Temp]);
}

void LCDSendInt(long a)
{
	int C[20];
	unsigned char Temp=0, NumLen = 0;
	if (a < 0)
	{
		LCDSendChar('-');
		a = -a;
	}
	do
	{	
		Temp++;
		C[Temp] = a % 10;
		a = a/10;
	}
	while (a);
	NumLen = Temp;
	for (Temp = NumLen; Temp>0; Temp--) LCDSendChar(C[Temp] + 48);
}

/**********************************************************L C D*******************************************************/

void Value_Init()
{
	Freq[0] = 10;
	Freq[1] = 500;
}

void Port_Init()
{
	PORTA = 0b00000000;		DDRA = 0b01000000;	//set Relay as output (Bit6 = 1)
	PORTB = 0b00000000;		DDRB = 0b00000000;
	PORTC = 0b00000000;		DDRC = 0b11110111;	//set Display ports (D7-D4, E, R/W, RS) as output
	PORTD = 0b11000000;		DDRD = 0b00001000;	//set TX as output
	PORTE = 0b00000000;		DDRE = 0b00110000;	//set Buzzer's ports as output
	PORTF = 0b00000000;		DDRF = 0b00000000;
	PORTG = 0b00000000;		DDRG = 0b00000000;
}

void Print_Display()
{
	switch (Line)
	{
		case 1: 
				LCDSendTxt("BLINK: 1-20     ");
				LCDSendCommand(DD_RAM_ADDR2);
				LCDSendTxt("SPEED: ");
				LCDSendInt(Freq[Line - 1]);
				LCDSendTxt("  ");
			break;
		case 2: 
				LCDSendTxt("BUZZER: 1-30    ");
				LCDSendCommand(DD_RAM_ADDR2);
				LCDSendTxt("TUNE: ");
				LCDSendInt(Freq[Line - 1] / 100);
				LCDSendTxt("  ");
			break;
	}
	LCDSendCommand(DD_RAM_ADDR);
}

void Buttons()
{
		//Bl prevent holding button

		//Up Button (Button 1)
		//switch between Relay and Buzzer
		if ((!(PINA & 0b00000001)) & Bl)
		{
			if (Line == 1)
				Line = 2;
			else
				Line = 1;
			Bl = 0;
		}

		//Left Button (Button 2)
		//Decrease frequency of Relay/Buzzer
		if ((!(PINA & 0b00000010)) & Bl)
		{
			if (Line == 1)
				Freq[0]--;
			else
				Freq[1] = Freq[1] - 100;
			if (Freq[0] < 1) Freq[0] = 1;
			if (Freq[1] < 100) Freq[1] = 100;
			Bl = 0;
		}

		//Right Button (Button 4)
		//Increase frequency of Relay/Buzzer
		if ((!(PINA & 0b00001000)) & Bl)
		{
			if (Line == 1)
				Freq[0]++;
			else
				Freq[1] = Freq[1] + 100;
			if (Freq[0] > 20) Freq[0] = 20;
			if (Freq[1] > 3000) Freq[1] = 3000;
			Bl = 0;
		}

		//Down Button (Button 5)
		//switch between Relay and Buzzer
		if ((!(PINA & 0b00010000)) & Bl)
		{
			if (Line == 1)
				Line = 2;
			else
				Line = 1;
			Bl = 0;
		}

		//check for release of ALL buttons
		if (
			((PINA & 0b00000001)
			|(PINA & 0b00000010)
			|(PINA & 0b00001000)
			|(PINA & 0b00010000)) == 27)
		Bl = 1;
}

void Buzzer()
{
	unsigned int i;
	while ((!(PINA & 0b00000100)) & Line == 2)		//while Button3(Middle) is pressed and activated buzzer (Line=2)
	{
		Buttons();									//check for pressing another button
		//Print_Display();
		if ((!(PINA & 0b00000100)) & Line == 2)		//check Button3(Middle) is pressed and activated buzzer (Line=2)
		PORTE = (PORTE & 0b11011111) | 0b00010000;	//switch states of bit4 = 0 and bit5 = 1
		for (i=Freq[1]; i; i--);					//delay
		Buttons();									//check for pressing another button
		//Print_Display();
		if ((!(PINA & 0b00000100)) & Line == 2)		//check Button3(Middle) is pressed and activated buzzer (Line=2)
		PORTE = (PORTE | 0b00100000) & 0b11101111;	//switch states of bit4 = 0 and bit5 = 1
		for (i=Freq[1]; i; i--);					//delay
	}
}

void Blink()
{
	unsigned int i;
	while ((!(PINA & 0b00000100)) & Line == 1)		//while Button3(Middle) is pressed and activated relay (Line=1)
	{
		Buttons();									//check for pressing another button
		Print_Display();							//reprint the display while blinking
		if ((!(PINA & 0b00000100)) & Line == 1)		//check Button3(Middle) is pressed and activated relay (Line=1)
		PORTA = PORTA & 0b10111111;					//switch state of relay (Bit6 = 0)
		if ((!(PINA & 0b00000100)) & Line == 1)		//check Button3(Middle) is pressed and activated relay (Line=1)
		for (i=Freq[0]; i; i--)						//delay
		{
			Buttons();								//check for pressing another button
			Print_Display();						//reprint the display while blinking
		}
		if ((!(PINA & 0b00000100)) & Line == 1)		//check Button3(Middle) is pressed and activated relay (Line=1)
		PORTA = PORTA | 0b01000000;					//switch state of relay (Bit6 = 1)
		if ((!(PINA & 0b00000100)) & Line == 1)		//check Button3(Middle) is pressed and activated relay (Line=1)
		for (i=Freq[0]; i; i--)
		{
			Buttons();								//check for pressing another button
			Print_Display();						//reprint the display while blinking
		}
	}
}

int main()
{
	Port_Init();
	LCD_Init();
	Value_Init();
	LCDSendCommand(DISP_ON);
	LCDSendCommand(CLR_DISP);
	while (1) 
	{
		Buttons();
		Print_Display();

		//Middle Button (Button 3)
		//activating the Relay/Buzzer
		if (!(PINA & 0b00000100))
		{
			if (Line == 1)
				Blink();
			else
				Buzzer();
		}
	}
	return 0;
}
